//**********************************************************************
// CRhsEventLogReader.cpp
// ----------------------
// The CRhsEventLogReader class is a thin wrapper round the
// CRhsEventLog class. Most methods simply call the corresponding
// CRhsEventLog method then do whatever data mangling is required by
// COM.
//
// John Rennie
// john.rennie@ratsauce.co.uk
// 20th December 2006
//**********************************************************************

#include "stdafx.h"
#include "RhsEventLogReader.h"
#include "CRhsEventLogReader.h"


//**********************************************************************
// Prototypes
// ----------
// These are helper functions used by the class.
//**********************************************************************

double DateTimeToDouble(SYSTEMTIME* st);
int DaysPerMonth(int Month, int Year);


//**********************************************************************
// CRhsEventLogReader::OpenLog
// ---------------------------
// Open the connection to the event log and load the first record.
// Type is one of "System", "Application" or "Security".
// ReadBack = 0 means start at the oldest record, non-zero means start
// at the most recent record.
// If Computer is not "" the event log on the computer with this name
// will be opened.
// The result is set to zero if the attempt to open the log fails or
// non-zero if it succeeds.
//**********************************************************************

STDMETHODIMP CRhsEventLogReader::OpenLog(BSTR Type, int ReadBack, BSTR Computer, long * Result)
{ char type[256], computer[256];

// If there is already a connection open delete the event log object to
// close the connection then create a new object.

  if (m_EventLog)
  { m_EventLog->CloseLog();
    delete m_EventLog;
    m_EventLog = NULL;
  }

  m_EventLog = new CRhsEventLog;

// Convert the arguments to char

  WideCharToMultiByte(CP_ACP, 0, Type,     -1, type,     255, NULL, NULL);
  WideCharToMultiByte(CP_ACP, 0, Computer, -1, computer, 255, NULL, NULL);

// Open the log

  *Result = (long) m_EventLog->OpenLog(type, ReadBack, computer);

// All done

  return S_OK;
}


//**********************************************************************
// CRhsEventLogReader::CloseLog
// ----------------------------
// Close the connection to the event log.
//**********************************************************************

STDMETHODIMP CRhsEventLogReader::CloseLog(long * Result)
{

// If the log is open close it and free our RhsEventLog object

  if (m_EventLog)
  { m_EventLog->CloseLog();
    delete m_EventLog;
    m_EventLog = NULL;
  }

// Always return TRUE

  *Result = TRUE;

// All done

  return S_OK;
}


//**********************************************************************
// CRhsEventLogReader::ReadRecord
// ------------------------------
// Read the next record from the currently open event log.
// Return zero if the record cannot be loaded.
//**********************************************************************

STDMETHODIMP CRhsEventLogReader::ReadRecord(long * Result)
{

// Check our RhsEventLog object exists

  if (!m_EventLog)
  { *Result = 0;
    return S_OK;
  }

// Read the next record

  *Result = m_EventLog->ReadRecord(0);

// All done

  return S_OK;
}


//**********************************************************************
// CRhsEventLogReader::EventType
// -----------------------------
// Return the event type as a string i.e. one of:
// "Information"
// "Warning"
// "Error"
// Return "" if there is an error.
//**********************************************************************

STDMETHODIMP CRhsEventLogReader::EventType(BSTR * Result)
{ char type[256];
  WCHAR w_type[256];

// Set the return string to ""; this will be returned if there is an error

  w_type[0] = 0;

// Check our RhsEventLog object exists

  if (!m_EventLog)
  { *Result = SysAllocString(w_type);
    return S_OK;
  }

// Check a record is available

  if (!m_EventLog->RecordAvailable())
  { *Result = SysAllocString(w_type);
    return S_OK;
  }

// Return the event type

  if (!m_EventLog->Category())
  { strcpy(type, "");
  }
  else
  { strncpy(type, m_EventLog->Category(), 255);
    type[255] = '\0';
  }

  MultiByteToWideChar(CP_ACP, 0, type, -1, w_type, 255);
  *Result = SysAllocString(w_type);

// All done

  return S_OK;
}


//**********************************************************************
// CRhsEventLogReader::EventTime
// -----------------------------
// Return the event timestamp as a double with the date in VB format,
// i.e. the number of days since 1st Jan 1900.
//**********************************************************************

STDMETHODIMP CRhsEventLogReader::EventTime(double * Result)
{ SYSTEMTIME st;

// Check our RhsEventLog object exists

  if (!m_EventLog)
  { *Result = 0;
    return S_OK;
  }

// Check a record is available

  if (!m_EventLog->RecordAvailable())
  { *Result = 0;
    return S_OK;
  }

// Get the time as a systemtime then convert it to a double

  if (!m_EventLog->Time(&st))
  { *Result = 0;
    return S_OK;
  }

  *Result = DateTimeToDouble(&st);

// All done

  return S_OK;
}


//**********************************************************************
// CRhsEventLogReader::Source
// --------------------------
// Return the event source.
// Return "" if there is an error.
//**********************************************************************

STDMETHODIMP CRhsEventLogReader::Source(BSTR * Result)
{ char source[256];
  WCHAR w_source[256];

// Set the return string to ""; this will be returned if there is an error

  w_source[0] = 0;

// Check our RhsEventLog object exists

  if (!m_EventLog)
  { *Result = SysAllocString(w_source);
    return S_OK;
  }

// Check a record is available

  if (!m_EventLog->RecordAvailable())
  { *Result = SysAllocString(w_source);
    return S_OK;
  }

// Return the source

  if (!m_EventLog->Source())
  { strcpy(source, "");
  }
  else
  { strncpy(source, m_EventLog->Source(), 255);
    source[255] = '\0';
  }

  MultiByteToWideChar(CP_ACP, 0, source, -1, w_source, 255);
  *Result = SysAllocString(w_source);

// All done

  return S_OK;
}


//**********************************************************************
// CRhsEventLogReader::EventID
// ---------------------------
// Return the event ID.
// return 0 if there is an error.
//**********************************************************************

STDMETHODIMP CRhsEventLogReader::EventID(long * Result)
{

// Check our RhsEventLog object exists

  if (!m_EventLog)
  { *Result = FALSE;
    return S_OK;
  }

// Check a record is available

  if (!m_EventLog->RecordAvailable())
  { *Result = FALSE;
    return S_OK;
  }

// Return the event id. There is some junk in the top WORD of the ID
// that we remove by ANDing it with 0xFFFF.

  *Result = (long) (m_EventLog->EventID() & 0xFFFF);

// All done

  return S_OK;
}


//**********************************************************************
// CRhsEventLogReader::Message
// ---------------------------
// Return the event text.
// Note this may contain carriage returns and line feeds.
// Return "" if there is an error.
//**********************************************************************

STDMETHODIMP CRhsEventLogReader::Message(BSTR * Result)
{ char msg[1024];
  WCHAR w_msg[1024];

// Set the return string to ""; this will be returned if there is an error

  w_msg[0] = 0;

// Check our RhsEventLog object exists

  if (!m_EventLog)
  { *Result = SysAllocString(w_msg);
    return S_OK;
  }

// Check a record is available

  if (!m_EventLog->RecordAvailable())
  { *Result = SysAllocString(w_msg);
    return S_OK;
  }

// Return the source

  if (!m_EventLog->Message())
  { strcpy(msg, "");
  }
  else
  { strncpy(msg, m_EventLog->Message(), 1023);
    msg[1023] = '\0';
  }

  MultiByteToWideChar(CP_ACP, 0, msg, -1, w_msg, 1023);
  *Result = SysAllocString(w_msg);

// All done

  return S_OK;
}


//**********************************************************************
// DateTimeToDouble
// ----------------
// Helper function to convert a SYSTEMTIME to a VB format double
//**********************************************************************

double DateTimeToDouble(SYSTEMTIME* st)
{ int i;
  double d;

  d = ((st->wYear - 1900)/4)*1461;

  if (st->wYear % 4 > 0)
    d += (st->wYear % 4)*365 + 1;

  for (i = 1; i < st->wMonth; i++)
    d += DaysPerMonth(i, st->wYear);

  d += st->wDay;

  d += (st->wHour + (st->wMinute + st->wSecond/60.0)/60.0)/24.0;

  return(d);
}


int DaysPerMonth(int Month, int Year)
{ static int days_per_month[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

  if (Month < 1 || Month > 12)
  { return(0);
  }
  else if (Month == 2 && Year % 4 == 0)
  { return(29);
  }
  else
  { return(days_per_month[Month-1]);
  }
}


